import inspect
import re
from typing import get_type_hints

from maitai_gen.chat import Function, Tool


def tool(func):
    function_description = func.__doc__
    if not function_description:
        raise ValueError("Missing docstring for tool")

    def wrapper(*args, **kwargs):
        return func(*args, **kwargs)

    sig = inspect.signature(func)
    params = sig.parameters

    properties = {}
    required = []
    type_hints = get_type_hints(func)

    param_descriptions = _extract_param_details(func.__doc__)
    for param_name, param in params.items():
        if param_name in type_hints:
            param_type = type_hints[param_name].__name__
        elif param.default is not param.empty and param.default is not None:
            param_type = type(param.default).__name__
        else:
            param_type = "string"
        properties[param_name] = {
            "type": _parse_type(param_type),
            "description": param_descriptions.get(param_name, ""),
        }
        if param.default is param.empty:
            required.append(param_name)

    wrapper.__tool__ = Tool(
        type="function",
        function=Function(
            name=func.__name__,
            description=function_description,
            parameters={
                "type": "object",
                "properties": properties,
                "required": required,
            },
        )
    )
    return wrapper


def _extract_param_details(docstring):
    param_descriptions = {}
    param_pattern = re.compile(r":param (\w+): (.+)")
    args_pattern = re.compile(r"Args:\s*(.*?)(?=\n\S|$)", re.DOTALL)
    arg_param_pattern = re.compile(r"(\w+): (.+)")

    args_match = args_pattern.search(docstring)
    if args_match:
        args_section = args_match.group(1)
        arg_lines = args_section.split("\n")
        for line in arg_lines:
            if line.strip():
                param_match = arg_param_pattern.match(line.strip())
                if param_match:
                    param_name, param_description = param_match.groups()
                    param_descriptions[param_name] = param_description.strip()
    param_matches = param_pattern.findall(docstring)
    for param, description in param_matches:
        param_descriptions[param] = description.strip()
    return param_descriptions


def _parse_type(type_string):
    if type_string == "str" or type_string == "string":
        return "string"
    if type_string == "int" or type_string == "integer":
        return "integer"
    if type_string == "bool" or type_string == "boolean":
        return "boolean"
    if type_string == "float" or type_string == "number":
        return "number"
    return "string"
